package com.fundi.framework.common.cslcomms;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.Serializable;
import java.util.ArrayList;

import com.fundi.framework.common.cslcomms.CSLFieldDefn;
import com.fundi.framework.common.cslcomms.CSLObject;
import com.fundi.framework.common.displaylist.ListField;
import com.fundi.framework.common.displaylist.ListFieldDefn;
import com.fundi.framework.common.displaylist.ListRow;
import com.fundi.framework.common.properties.ISelectionSource;
import com.fundi.framework.common.properties.PropertyDefn;

public class CSLBasedObject implements Serializable, ISelectionSource {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;
	
	private CSLObject sceObj = null;
	
	private ArrayList<ListField> fields = new ArrayList<ListField>();
	
	public CSLBasedObject(CSLObject sceObj) {
		
		super();
		
		int idx;
		ArrayList<CSLFieldDefn> sceDefns;
		CSLFieldDefn defn;
		
		this.sceObj = sceObj;
		
		sceDefns = sceObj.getDefns();
		for (idx = 0; idx < sceDefns.size(); idx++) {
			defn = sceDefns.get(idx);
			fields.add(new ListField(new ListFieldDefn(defn), sceObj.getFieldValueByLlbl(defn.getLlbl()).trim()));
		}
	}
	
	public ListRow getAsListRow() {
		
		ListRow row = new ListRow();
		int idx;
		
		for (idx = 0; idx < fields.size(); idx++) {
			row.addField(fields.get(idx));
			row.setData(this);
		}
		
		return row;
	}

	@Override
	public PropertyDefn[] getPropertyDefns() {
		
		PropertyDefn[] defns = new PropertyDefn[fields.size()];
		int idx;
		int idxShow;
		ListField fld;
		
		idxShow = 0;
		for (idx = 0; idx < fields.size(); idx++) {
			if (fields.get(idx).isVisible()) {
				idxShow++;
			}
		}
		defns = new PropertyDefn[idxShow];
		idxShow = -1;
		for (idx = 0; idx < fields.size(); idx++) {
			fld = fields.get(idx);
			if (fld.isVisible()) {
				defns[++idxShow] = new PropertyDefn(fld.getLabel(), fld.getLabel());
			}
		}

		return defns;
	}

	@Override
	public String getPropertyValue(String id) {

		int idx;
		ListField fld;
		
		for (idx = 0; idx < fields.size(); idx++) {
			fld = fields.get(idx);
			if (fld.getLabel().equals(id)) {
				return fld.getValue();
			}
		}
		
		return null;
	}
	
	public void setPropertyValue(String id, String value) {
		
		int idx;
		ListField fld;
		
		for (idx = 0; idx < fields.size(); idx++) {
			fld = fields.get(idx);
			if (fld.getLabel().equals(id)) {
				fld.setValue(value);
			}
		}
	}

	public CSLObject getSource() {
		
		return sceObj;
	}
	
	public Object getProvider() {
		
		return sceObj.getProvider();
	}
}
