package com.fundi.framework.common.displaylist;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.Serializable;

import javax.xml.bind.annotation.XmlRootElement;

import com.fundi.framework.common.cslcomms.CSLFieldDefn;
import com.fundi.framework.common.nl1.Messages;

@XmlRootElement
public class ListFieldDefn implements Serializable {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;
	
	public static final String alignLeft = Messages.getString("ListFieldDefn_4"); //$NON-NLS-1$
	public static final String alignRight = Messages.getString("ListFieldDefn_5"); //$NON-NLS-1$
	public static final String alignCentre = Messages.getString("ListFieldDefn_6"); //$NON-NLS-1$
	public static final String typeCharacter = Messages.getString("ListFieldDefn_7"); //$NON-NLS-1$
	public static final String typeNumeric = Messages.getString("ListFieldDefn_8"); //$NON-NLS-1$
	public static final String typeBoolean = Messages.getString("ListFieldDefn_9"); //$NON-NLS-1$
	
	private String name = ""; //$NON-NLS-1$
	private String label = ""; //$NON-NLS-1$
	private String dataType = typeCharacter;
	private String alignment = alignLeft;
	private boolean hiddenByUser = false;	//MA: moved this into its own variable as users mustn't be shown server_hidden fields at all
	private boolean visible = true;		//MA: need to keep the visible property for compatibility with saved displays, but treating it as server_visible
	private boolean hiddenForBlankValue = false;
	private boolean hiddenForZeroValue = false;
	private boolean hiddenForSameValue = false;
	private String alias = ""; //$NON-NLS-1$
	private String compareLabel = ""; //$NON-NLS-1$
	private boolean compareIgnore = false;
	private boolean compareHide = false;
	
	private boolean key = false;
	private int keySeq = 0;
	
	private boolean inAllRows = true;
	
	private PropDefn propDefn = null;
	
	public ListFieldDefn() {
	
		super();
	}
	
	public ListFieldDefn(String name, String type, String align) {
		
		initialise(name, type, align, false, true);
	}

	public ListFieldDefn(String name, String type, String align, boolean key) {
		
		initialise(name, type, align, key, true);
	}
	
	public ListFieldDefn(String name, String type, String align, boolean key, boolean visible) {
		
		initialise(name, type, align, key, visible);
	}
	
	public ListFieldDefn(String name, String type, String align, boolean key, boolean visible, int keySeq) {
		
		initialise(name, type, align, key, visible);
		setKey(keySeq);
	}
	
	public ListFieldDefn(ListFieldDefn sourceDefn) {
		
		//This constructor is designed for Comparator use.
		name = sourceDefn.getKeyLabel();
		label = sourceDefn.getLabel();
		dataType = sourceDefn.getDataType();
		alignment = sourceDefn.getAlignment();
		hiddenByUser = sourceDefn.isHiddenByUser();
		visible = !sourceDefn.isHiddenByServer();
		compareLabel = sourceDefn.getCompareLabel();
	}
	
	public ListFieldDefn(CSLFieldDefn cslDefn) {
		
		super();
		
		setKeyLabel(cslDefn.getKeyLabel());
		label = cslDefn.getLlbl();
		if (cslDefn.getAlign().equals(CSLFieldDefn.ALIGN_RIGHT)) {
			setAlignment(ListFieldDefn.alignRight);
		}
		if (cslDefn.getDtype().equals(CSLFieldDefn.DTYPE_INT)) {
			setDataType(ListFieldDefn.typeNumeric);
		}
		setKey(cslDefn.getKeySeq());
		setCompareIgnore(cslDefn.ignoreInCompare());
		setCompareHide(cslDefn.hideInCompare());
		if (cslDefn.hideField()) {  //Done this way in case the visibility was set to true some other way.
			setHiddenByServer(true);
		}
	}
	
	public ListFieldDefn(PropDefn pdefn) {
		String	type;

		if (PropDefn.displayFormatBoolean.equals(pdefn.getDisplayFormat())) {
			type = typeBoolean;
		} else if (PropDefn.displayFormatInteger.equals(pdefn.getDisplayFormat())) {
			type = typeNumeric;
		} else {
			type = typeCharacter;
		}
		initialise(
				pdefn.getKeyLabel(),
				type, 
				PropDefn.displayFormatInteger.equals(pdefn.getDisplayFormat()) ? alignRight : alignLeft, 
				pdefn.isKey(),
				true
		);
		label = pdefn.getDisplayName();
		propDefn = pdefn;
	}
	
	private void initialise(String name, String type, String align, boolean key, boolean visible) {
		
		this.name = name;
		this.dataType = type;
		this.alignment = align;
		this.key = key;
		if (key) {
			keySeq = 1;
		}
		else {
			keySeq = 0;
		}
		this.visible = visible;
		
		label = name;  //Default the label to the name.
	}

	public String getLabel() {
		
		if (alias.length() == 0) {
			return label;
		}
		else {
			return alias;
		}
	}

	/**
	 * Use setKeyLabel instead. Changed to align the API with the new PropDefn functionality.
	 */
	@Deprecated
	public void setName(String name) {
		
		this.name = name;
		label = name;  //Default the label to the name;
	}
	
	public void setKeyLabel(String name) {
		
		this.name = name;
		label = name;  //Default the label to the name - though the intention is that this will always get overwritten such that the keylabel never displays to the user;
	}
	
	public String getDataType() {
		return dataType;
	}

	public void setDataType(String dataType) {
		this.dataType = dataType;
	}

	public String getAlignment() {
		return alignment;
	}

	public void setAlignment(String alignment) {
		this.alignment = alignment;
	}

	public boolean isKey() {
		return (key && keySeq > 0);
	}

	public int getKeySeq() {
		
		return keySeq;
	}

	public void setKey(boolean key) {
		this.key = key;
		if (key) {
			keySeq = 1;
		}
		else {
			keySeq = 0;
		}
	}

	public void setKey(int keySeq) {
		
		key = (keySeq > 0);
		this.keySeq = keySeq;
	}

	public boolean isVisible() {
		return (visible && !hiddenByUser && !hiddenForBlankValue && !hiddenForZeroValue && !hiddenForSameValue);
	}

	public void setHiddenForBlankValue(boolean hidden) {
		
		hiddenForBlankValue = hidden;
	}

	public void setHiddenForZeroValue(boolean hidden) {
		
		hiddenForZeroValue = hidden;
	}

	public void setHiddenForSameValue(boolean hidden) {
		
		hiddenForSameValue = hidden;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.toLowerCase().hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ListFieldDefn other = (ListFieldDefn) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equalsIgnoreCase(other.name))
			return false;
		return true;
	}

	public String getAlias() {
		
		return alias;
	}

	public void setAlias(String alias) {
		
		this.alias = alias;
	}

	/**
	 * Use getKeyLabel instead. Changed to align the API with the new PropDefn functionality.
	 * @return the keyLabel for this field. May match LLBL but this is not guaranteed.
	 */
	@Deprecated
	public String getName() {
		return name;
	}

	public String getKeyLabel() {
		return name;
	}

	public boolean isInAllRows() {
		return inAllRows;
	}

	public void setInAllRows(boolean inAllRows) {
		this.inAllRows = inAllRows;
	}

	public void setCompareLabel(String compareLabel) {
		
		this.compareLabel = compareLabel;
	}

	public String getCompareLabel() {
		
		if (compareLabel == null) {
			compareLabel = ""; //$NON-NLS-1$
		}
		
		if (compareLabel.length() == 0) {
			return getLabel();
		}
		else {
			return compareLabel;
		}
	}

	public boolean ignoreInCompare() {
		
		return compareIgnore;
	}
	
	public void setCompareIgnore(boolean ignore) {
		
		compareIgnore = ignore;
	}
	
	public boolean hideInCompare() {
		
		return compareHide;
	}
	
	public void setCompareHide(boolean hide) {
		
		compareHide = hide;
	}
	
	public String toString() {
		StringBuilder result = new StringBuilder();
		
		result.append(getLabel());	//want to be exercising this method in case the logic ever changes within it.
		if (alias != null && !alias.isEmpty()) {
			result.append(":"); //$NON-NLS-1$
			result.append(label);
		}
		
		return result.toString();
	}

	public boolean isHiddenByUser() {
		return hiddenByUser;
	}

	public void setHiddenByUser(boolean hiddenByUser) {
		this.hiddenByUser = hiddenByUser;
	}

	public boolean isHiddenByServer() {
		return !visible;
	}

	public void setHiddenByServer(boolean hiddenByServer) {
		this.visible = !hiddenByServer;
	}

	public PropDefn getPropDefn() {
		return propDefn;
	}

	public void setPropDefn(PropDefn propDefn) {
		this.propDefn = propDefn;
	}
	
	public String getEditKeyLabel() {
		
		if (propDefn != null) {
			return propDefn.getKeyLabel();
		}
		return null;
	}

	public String getEditDisplayName() {

		if (propDefn != null) {
			return propDefn.getDisplayName();
		}
		return null;
	}

	public String getEditDisplayFormat() {

		if (propDefn != null) {
			return propDefn.getDisplayFormat();
		}
		return null;
	}

	public String getTip() {

		if (propDefn != null) {
			return propDefn.getTip();
		}
		return null;
	}

	public int getEditLength() {

		if (propDefn != null) {
			return propDefn.getLength();
		}
		return PropDefn.defaultLength;
	}

	public String getEditDefaultValue() {

		if (propDefn != null) {
			return propDefn.getDefaultValue();
		}
		return null;
	}

	public boolean isRequired() {

		if (propDefn != null) {
			return propDefn.isRequired();
		}
		return PropDefn.defaultRequired;
	}

	public boolean isEditable() {

		if (propDefn != null) {
			return propDefn.isEditable();
		}
		return PropDefn.defaultEditable;
	}

	public boolean isEditKey() {

		if (propDefn != null) {
			return propDefn.isKey();
		}
		return PropDefn.defaultKey;
	}

	public boolean isEditFormHide() {

		if (propDefn != null) {
			return propDefn.isFormHide();
		}
		return PropDefn.defaultFormHide;
	}

	public String getEditValueTrue() {

		if (propDefn != null) {
			return propDefn.getValueTrue();
		}
		return null;
	}

	public String getEditValueFalse() {

		if (propDefn != null) {
			return propDefn.getValueFalse();
		}
		return null;
	}
}
